@@ -88,10 +88,11 @@ class CliParser(object):
         opts = {}
         if os.path.exists("./harpoon.yml"):
             opts["default"] = "./harpoon.yml"
+        else:
+            opts["required"] = True
         parser.add_argument("--harpoon-config"
             , help = "The config file specifying what harpoon should care about"
             , type = argparse.FileType("r")
-            , required = True
             , **opts
             )
 
@@ -147,6 +148,12 @@ class CliParser(object):
             , action = "append"
             )
 
+        parser.add_argument("--port"
+            , help = "Specify a port to publish in the running container you make"
+            , dest = "ports"
+            , action = "append"
+            )
+
         return parser
 
 def docker_context():
