@@ -1,3 +1,5 @@
+from __future__ import print_function
+
 from harpoon.errors import NoSuchKey, BadOption, NoSuchImage, BadCommand, BadImage, ProgrammerError, HarpoonError, FailedImage, BadResult
 from harpoon.formatter import MergedOptionStringFormatter
 from harpoon.helpers import a_temp_file, until
@@ -70,10 +72,13 @@ class Image(object):
 
         Taking into account parent image, and those in link and volumes_from options
         """
+        candidates = []
+        detach = dict((candidate, not options.get("attached", False)) for candidate, options in self.dependency_options.items())
+
         if not ignore_parent:
             for image, instance in images.items():
                 if self.parent_image == instance.image_name:
-                    yield image
+                    candidates.append(image)
                     break
 
         container_names = dict((instance.container_name, image) for image, instance in images.items())
@@ -83,12 +88,19 @@ class Image(object):
                 if ":" in image:
                     image = image.split(":", 1)[0]
                 if image in container_names:
-                    yield container_names[image]
+                    name = container_names[image]
+                    candidates.append(container_names[image])
 
         if self.volumes_from:
             for image in self.volumes_from:
                 if image in container_names:
-                    yield container_names[image]
+                    candidates.append(container_names[image])
+
+        done = set()
+        for candidate in candidates:
+            if candidate not in done:
+                done.add(candidate)
+                yield candidate, detach.get(candidate, True)
 
     @property
     def container_id(self):
@@ -181,17 +193,17 @@ class Image(object):
                             result[port] = port
         return result
 
-    def run_container(self, images, detach=False, command=None, started=None, extra_env=None, extra_volumes=None, extra_ports=None):
+    def run_container(self, images, detach=False, command=None, started=None, extra_env=None, extra_volumes=None, extra_ports=None, dependency=False):
         """Run this image and all dependency images"""
         if self.already_running:
             return
 
         try:
-            for dependency in self.dependency_images(images, ignore_parent=True):
+            for dependency_name, detached in self.dependency_images(images, ignore_parent=True):
                 try:
-                    images[dependency].run_container(images, detach=True)
+                    images[dependency_name].run_container(images, detach=detached, dependency=True)
                 except Exception as error:
-                    raise BadImage("Failed to start dependency container", image=self.name, dependency=dependency, error=error)
+                    raise BadImage("Failed to start dependency container", image=self.name, dependency=dependency_name, error=error)
 
             env = self.figure_out_env(extra_env)
             ports = self.figure_out_ports(extra_ports)
@@ -206,19 +218,21 @@ class Image(object):
             volumes_from = self.volumes_from
             self._run_container(self.name, self.image_name, self.container_name
                 , detach=detach, command=command, tty=tty, env=env, ports=ports
-                , volumes=volumes, volumes_from=volumes_from, links=links
+                , volumes=volumes, volumes_from=volumes_from, links=links, dependency=dependency
                 )
 
         finally:
-            if not detach:
-                for dependency in self.dependency_images(images, ignore_parent=True):
+            if not detach and not dependency:
+                for dependency, _ in self.dependency_images(images, ignore_parent=True):
                     try:
-                        images[dependency].stop_container()
+                        images[dependency].stop_container(fail_on_bad_exit=True, fail_reason="Failed to run dependency container")
+                    except BadImage:
+                        raise
                     except Exception as error:
                         log.warning("Failed to stop dependency container\timage=%s\tdependency=%s\tcontainer_name=%s\terror=%s", self.name, dependency, images[dependency].container_name, error)
                 self.stop_container()
 
-    def _run_container(self, name, image_name, container_name, detach=False, command=None, tty=True, volumes=None, volumes_from=None, links=None, delete_on_exit=False, env=None, ports=None):
+    def _run_container(self, name, image_name, container_name, detach=False, command=None, tty=True, volumes=None, volumes_from=None, links=None, delete_on_exit=False, env=None, ports=None, dependency=False):
         """Run a single container"""
         log.info("Creating container from %s\timage=%s\tcontainer_name=%s\ttty=%s", image_name, name, container_name, tty)
 
@@ -287,11 +301,14 @@ class Image(object):
                 , port_bindings = ports
                 )
 
-            if not detach:
-                dockerpty.start(self.docker_context, container_id)
+            if not detach and not dependency:
+                try:
+                    dockerpty.start(self.docker_context, container_id)
+                except KeyboardInterrupt:
+                    pass
 
             inspection = None
-            if not detach:
+            if not detach and not dependency:
                 for _ in until(timeout=0.5, step=0.1, silent=True):
                     try:
                         inspection = self.docker_context.inspect_container(container_id)
@@ -304,22 +321,30 @@ class Image(object):
 
             if inspection:
                 if not inspection["State"]["Running"] and inspection["State"]["ExitCode"] != 0:
-                    raise BadImage("Failed to run container", container_id=container_id)
+                    if self.interactive and not self.heira_formatted("harpoon.no_intervention", default=False):
+                        print("!!!!")
+                        print("Failed to run the container!")
+                        print("Do you want commit the container in it's current state and /bin/bash into it to debug?")
+                        answer = raw_input("[y]: ")
+                        if not answer or answer.lower().startswith("y"):
+                            with self.commit_and_run(container_id, command="/bin/bash"):
+                                pass
+                    raise BadImage("Failed to run container", container_id=container_id, container_name=container_name)
         finally:
             if delete_on_exit:
                 self._stop_container(container_id, container_name)
 
-    def stop_container(self):
+    def stop_container(self, fail_on_bad_exit=False, fail_reason=None):
         """Stop this container if it exists"""
         container_id = self.container_id
         if container_id is None:
             return
 
-        self._stop_container(container_id, self.container_name)
+        self._stop_container(container_id, self.container_name, fail_on_bad_exit=fail_on_bad_exit, fail_reason=fail_reason)
         self._container_id = None
         self.already_running = False
 
-    def _stop_container(self, container_id, container_name):
+    def _stop_container(self, container_id, container_name, fail_on_bad_exit=False, fail_reason=None):
         """Stop some container"""
         stopped = False
         for _ in until(timeout=10):
@@ -341,7 +366,26 @@ class Image(object):
                 else:
                     break
 
-        if not stopped:
+        if stopped:
+            exit_code = inspection["State"]["ExitCode"]
+            if exit_code != 0 and fail_on_bad_exit:
+                if not self.interactive:
+                    print_logs = True
+                else:
+                    print("!!!!")
+                    print("Container had already exited with a non zero exit code\tcontainer_name={0}\tcontainer_id={1}\texit_code={2}".format(container_name, container_id, exit_code))
+                    print("Do you want to see the logs from this container?")
+                    answer = raw_input("[y]: ")
+                    print_logs = not answer or answer.lower().startswith("y")
+
+                if print_logs:
+                    print("=================== Logs for failed container {0} ({1})".format(container_id, container_name))
+                    for line in self.docker_context.logs(container_id).split("\n"):
+                        print(line)
+                    print("------------------- End logs for failed container")
+                fail_reason = fail_reason or "Failed to run container"
+                raise BadImage(fail_reason, container_id=container_id, container_name=container_name)
+        else:
             try:
                 log.info("Killing container %s:%s", container_name, container_id)
                 self.docker_context.kill(container_id, 9)
@@ -568,12 +612,18 @@ class Image(object):
             yield
             return
 
+        with self.commit_and_run(container_id, command="/bin/bash"):
+            yield
+
+    @contextmanager
+    def commit_and_run(self, container_id, command="/bin/bash"):
+        """Commit this container id and run the provided command in it and clean up afterwards"""
         try:
             image = self.docker_context.commit(container_id)
             image_hash = image["Id"]
 
-            name = "{0}-intervention".format(self.name)
-            self._run_container(name, image_hash, image_hash, detach=False, tty=True, command="/bin/bash", delete_on_exit=True)
+            name = "{0}-intervention-{1}".format(container_id, str(uuid.uuid1()))
+            self._run_container(name, image_hash, image_hash, detach=False, tty=True, command=command, delete_on_exit=True)
             yield
         except Exception as error:
             log.error("Something failed about creating the intervention image\terror=%s", error)
@@ -677,6 +727,10 @@ class Image(object):
         self.extra_context = []
         self.commands = self.interpret_commands(self.configuration["commands"])
 
+        self.dependency_options = self.heira_formatted("dependency_options", default={})
+        if not isinstance(self.dependency_options, dict) and not isinstance(self.dependency_options, MergedOptions):
+            raise BadOption("Dependency options must be a dictionary", got=self.dependency_options)
+
     def normalise_volumes(self, volumes):
         """Return normalised version of these volumes"""
         result = []
@@ -802,7 +856,7 @@ class Imager(object):
         if image not in images:
             raise NoSuchImage(looking_for=image, available=images.keys())
 
-        for dependency in images[image].dependency_images(images):
+        for dependency, _ in images[image].dependency_images(images):
             self.make_image(dependency, chain=chain + [image], made=made)
 
         # Should have all our dependencies now
